@@ -6,7 +6,7 @@ use zellij_tile::prelude::*;
 use crate::line::tab_line;
 use crate::tab::tab_style;
 
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct LinePart {
     part: String,
     len: usize,
@@ -25,8 +25,6 @@ register_plugin!(State);
 impl ZellijPlugin for State {
     fn load(&mut self) {
         set_selectable(false);
-        set_invisible_borders(true);
-        set_fixed_height(1);
         subscribe(&[EventType::TabUpdate, EventType::ModeUpdate]);
     }
 
@@ -64,6 +62,7 @@ impl ZellijPlugin for State {
             all_tabs.push(tab);
         }
         let tab_line = tab_line(
+            self.mode_info.session_name.as_deref(),
             all_tabs,
             active_tab_index,
             cols,
