@@ -40,17 +40,12 @@ pub fn non_active_tab(text: String, palette: Palette, separator: &str) -> LinePa
 pub fn tab_style(
     text: String,
     is_active_tab: bool,
-    position: usize,
     is_sync_panes_active: bool,
     palette: Palette,
     capabilities: PluginCapabilities,
 ) -> LinePart {
     let separator = tab_separator(capabilities);
-    let mut tab_text = if text.is_empty() {
-        format!("Tab #{}", position + 1)
-    } else {
-        text
-    };
+    let mut tab_text = text;
     if is_sync_panes_active {
         tab_text.push_str(" (Sync)");
     }
