@@ -23,7 +23,17 @@ fn route_action(
     to_server: &SenderWithContext<ServerInstruction>,
 ) -> bool {
     let mut should_break = false;
+    session
+        .senders
+        .send_to_plugin(PluginInstruction::Update(None, Event::InputReceived))
+        .unwrap();
     match action {
+        Action::ToggleTab => {
+            session
+                .senders
+                .send_to_screen(ScreenInstruction::ToggleTab)
+                .unwrap();
+        }
         Action::Write(val) => {
             session
                 .senders
@@ -127,6 +137,12 @@ fn route_action(
                 .send_to_screen(ScreenInstruction::ScrollDownAt(point))
                 .unwrap();
         }
+        Action::ScrollToBottom => {
+            session
+                .senders
+                .send_to_screen(ScreenInstruction::ScrollToBottom)
+                .unwrap();
+        }
         Action::PageScrollUp => {
             session
                 .senders
@@ -145,6 +161,12 @@ fn route_action(
                 .send_to_screen(ScreenInstruction::ToggleActiveTerminalFullscreen)
                 .unwrap();
         }
+        Action::TogglePaneFrames => {
+            session
+                .senders
+                .send_to_screen(ScreenInstruction::TogglePaneFrames)
+                .unwrap();
+        }
         Action::NewPane(direction) => {
             let shell = session.default_shell.clone();
             let pty_instr = match direction {
@@ -175,11 +197,11 @@ fn route_action(
                 .send_to_screen(ScreenInstruction::CloseFocusedPane)
                 .unwrap();
         }
-        Action::NewTab => {
+        Action::NewTab(tab_layout) => {
             let shell = session.default_shell.clone();
             session
                 .senders
-                .send_to_pty(PtyInstruction::NewTab(shell))
+                .send_to_pty(PtyInstruction::NewTab(shell, tab_layout))
                 .unwrap();
         }
         Action::GoToNextTab => {
