@@ -5,23 +5,22 @@ use std::os::unix::io::RawFd;
 use std::str;
 use std::sync::{Arc, RwLock};
 
-use zellij_utils::zellij_tile;
+use zellij_utils::pane_size::Size;
+use zellij_utils::{input::layout::Layout, position::Position, zellij_tile};
 
 use crate::{
     panes::PaneId,
     pty::{PtyInstruction, VteBytes},
     tab::Tab,
     thread_bus::Bus,
-    ui::layout::Layout,
     wasm_vm::PluginInstruction,
     ServerInstruction, SessionState,
 };
-use zellij_tile::data::{Event, InputMode, ModeInfo, Palette, PluginCapabilities, TabInfo};
+use zellij_tile::data::{Event, ModeInfo, Palette, PluginCapabilities, TabInfo};
 use zellij_utils::{
     errors::{ContextType, ScreenContext},
-    input::options::Options,
+    input::{get_mode_info, options::Options},
     ipc::ClientAttributes,
-    pane_size::PositionAndSize,
 };
 
 /// Instructions that can be sent to the [`Screen`].
@@ -48,27 +47,32 @@ pub(crate) enum ScreenInstruction {
     MoveFocusRightOrNextTab,
     Exit,
     ScrollUp,
+    ScrollUpAt(Position),
     ScrollDown,
+    ScrollDownAt(Position),
+    ScrollToBottom,
     PageScrollUp,
     PageScrollDown,
     ClearScroll,
     CloseFocusedPane,
     ToggleActiveTerminalFullscreen,
-    SetSelectable(PaneId, bool),
-    SetFixedHeight(PaneId, usize),
-    SetFixedWidth(PaneId, usize),
-    SetInvisibleBorders(PaneId, bool),
+    TogglePaneFrames,
+    SetSelectable(PaneId, bool, usize),
     ClosePane(PaneId),
     ApplyLayout(Layout, Vec<RawFd>),
-    NewTab(RawFd),
     SwitchTabNext,
     SwitchTabPrev,
     ToggleActiveSyncTab,
     CloseTab,
     GoToTab(u32),
+    ToggleTab,
     UpdateTabName(Vec<u8>),
-    TerminalResize(PositionAndSize),
+    TerminalResize(Size),
     ChangeMode(ModeInfo),
+    LeftClick(Position),
+    MouseRelease(Position),
+    MouseHold(Position),
+    Copy,
 }
 
 impl From<&ScreenInstruction> for ScreenContext {
@@ -98,6 +102,7 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::Exit => ScreenContext::Exit,
             ScreenInstruction::ScrollUp => ScreenContext::ScrollUp,
             ScreenInstruction::ScrollDown => ScreenContext::ScrollDown,
+            ScreenInstruction::ScrollToBottom => ScreenContext::ScrollToBottom,
             ScreenInstruction::PageScrollUp => ScreenContext::PageScrollUp,
             ScreenInstruction::PageScrollDown => ScreenContext::PageScrollDown,
             ScreenInstruction::ClearScroll => ScreenContext::ClearScroll,
@@ -105,13 +110,10 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::ToggleActiveTerminalFullscreen => {
                 ScreenContext::ToggleActiveTerminalFullscreen
             }
+            ScreenInstruction::TogglePaneFrames => ScreenContext::TogglePaneFrames,
             ScreenInstruction::SetSelectable(..) => ScreenContext::SetSelectable,
-            ScreenInstruction::SetInvisibleBorders(..) => ScreenContext::SetInvisibleBorders,
-            ScreenInstruction::SetFixedHeight(..) => ScreenContext::SetFixedHeight,
-            ScreenInstruction::SetFixedWidth(..) => ScreenContext::SetFixedWidth,
             ScreenInstruction::ClosePane(_) => ScreenContext::ClosePane,
             ScreenInstruction::ApplyLayout(..) => ScreenContext::ApplyLayout,
-            ScreenInstruction::NewTab(_) => ScreenContext::NewTab,
             ScreenInstruction::SwitchTabNext => ScreenContext::SwitchTabNext,
             ScreenInstruction::SwitchTabPrev => ScreenContext::SwitchTabPrev,
             ScreenInstruction::CloseTab => ScreenContext::CloseTab,
@@ -120,6 +122,13 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::TerminalResize(_) => ScreenContext::TerminalResize,
             ScreenInstruction::ChangeMode(_) => ScreenContext::ChangeMode,
             ScreenInstruction::ToggleActiveSyncTab => ScreenContext::ToggleActiveSyncTab,
+            ScreenInstruction::ScrollUpAt(_) => ScreenContext::ScrollUpAt,
+            ScreenInstruction::ScrollDownAt(_) => ScreenContext::ScrollDownAt,
+            ScreenInstruction::LeftClick(_) => ScreenContext::LeftClick,
+            ScreenInstruction::MouseRelease(_) => ScreenContext::MouseRelease,
+            ScreenInstruction::MouseHold(_) => ScreenContext::MouseHold,
+            ScreenInstruction::Copy => ScreenContext::Copy,
+            ScreenInstruction::ToggleTab => ScreenContext::ToggleTab,
         }
     }
 }
@@ -134,13 +143,14 @@ pub(crate) struct Screen {
     /// A map between this [`Screen`]'s tabs and their ID/key.
     tabs: BTreeMap<usize, Tab>,
     /// The full size of this [`Screen`].
-    position_and_size: PositionAndSize,
+    size: Size,
     /// The index of this [`Screen`]'s active [`Tab`].
     active_tab_index: Option<usize>,
+    tab_history: Vec<Option<usize>>,
     mode_info: ModeInfo,
-    input_mode: InputMode,
     colors: Palette,
     session_state: Arc<RwLock<SessionState>>,
+    draw_pane_frames: bool,
 }
 
 impl Screen {
@@ -150,47 +160,23 @@ impl Screen {
         client_attributes: &ClientAttributes,
         max_panes: Option<usize>,
         mode_info: ModeInfo,
-        input_mode: InputMode,
         session_state: Arc<RwLock<SessionState>>,
+        draw_pane_frames: bool,
     ) -> Self {
         Screen {
             bus,
             max_panes,
-            position_and_size: client_attributes.position_and_size,
+            size: client_attributes.size,
             colors: client_attributes.palette,
             active_tab_index: None,
             tabs: BTreeMap::new(),
+            tab_history: Vec::with_capacity(32),
             mode_info,
-            input_mode,
             session_state,
+            draw_pane_frames,
         }
     }
 
-    /// Creates a new [`Tab`] in this [`Screen`], containing a single
-    /// [pane](crate::client::panes) with PTY file descriptor `pane_id`.
-    pub fn new_tab(&mut self, pane_id: RawFd) {
-        let tab_index = self.get_new_tab_index();
-        let position = self.tabs.len();
-        let tab = Tab::new(
-            tab_index,
-            position,
-            String::new(),
-            &self.position_and_size,
-            self.bus.os_input.as_ref().unwrap().clone(),
-            self.bus.senders.clone(),
-            self.max_panes,
-            Some(PaneId::Terminal(pane_id)),
-            self.mode_info.clone(),
-            self.input_mode,
-            self.colors,
-            self.session_state.clone(),
-        );
-        self.active_tab_index = Some(tab_index);
-        self.tabs.insert(tab_index, tab);
-        self.update_tabs();
-        self.render();
-    }
-
     /// Returns the index where a new [`Tab`] should be created in this [`Screen`].
     /// Currently, this is right after the last currently existing tab, or `0` if
     /// no tabs exist in this screen yet.
@@ -210,6 +196,8 @@ impl Screen {
         for tab in self.tabs.values_mut() {
             if tab.position == new_tab_pos {
                 tab.set_force_render();
+                self.tab_history.retain(|&e| e != Some(tab.index));
+                self.tab_history.push(self.active_tab_index);
                 self.active_tab_index = Some(tab.index);
                 break;
             }
@@ -229,6 +217,8 @@ impl Screen {
         for tab in self.tabs.values_mut() {
             if tab.position == new_tab_pos {
                 tab.set_force_render();
+                self.tab_history.retain(|&e| e != Some(tab.index));
+                self.tab_history.push(self.active_tab_index);
                 self.active_tab_index = Some(tab.index);
                 break;
             }
@@ -243,6 +233,8 @@ impl Screen {
         if let Some(t) = self.tabs.values_mut().find(|t| t.position == tab_index) {
             if t.index != active_tab_index {
                 t.set_force_render();
+                self.tab_history.retain(|&e| e != Some(t.index));
+                self.tab_history.push(self.active_tab_index);
                 self.active_tab_index = Some(t.index);
                 self.update_tabs();
                 self.render();
@@ -254,9 +246,6 @@ impl Screen {
     /// to be the last tab.
     pub fn close_tab(&mut self) {
         let active_tab_index = self.active_tab_index.unwrap();
-        if self.tabs.len() > 1 {
-            self.switch_tab_prev();
-        }
         let active_tab = self.tabs.remove(&active_tab_index).unwrap();
         let pane_ids = active_tab.get_pane_ids();
         // below we don't check the result of sending the CloseTab instruction to the pty thread
@@ -275,17 +264,22 @@ impl Screen {
                     .unwrap();
             }
         } else {
+            self.active_tab_index = self.tab_history.pop().unwrap();
             for t in self.tabs.values_mut() {
+                if t.position == self.active_tab_index.unwrap() {
+                    t.set_force_render()
+                }
                 if t.position > active_tab.position {
                     t.position -= 1;
                 }
             }
             self.update_tabs();
+            self.render();
         }
     }
 
-    pub fn resize_to_screen(&mut self, new_screen_size: PositionAndSize) {
-        self.position_and_size = new_screen_size;
+    pub fn resize_to_screen(&mut self, new_screen_size: Size) {
+        self.size = new_screen_size;
         for (_, tab) in self.tabs.iter_mut() {
             tab.resize_whole_tab(new_screen_size);
         }
@@ -320,6 +314,17 @@ impl Screen {
         }
     }
 
+    /// Returns an immutable reference to this [`Screen`]'s previous active [`Tab`].
+    /// Consumes the last entry in tab history.
+    pub fn get_previous_tab(&mut self) -> Option<&Tab> {
+        let last = self.tab_history.pop();
+        last?;
+        match last.unwrap() {
+            Some(tab) => self.tabs.get(&tab),
+            None => None,
+        }
+    }
+
     /// Returns a mutable reference to this [`Screen`]'s active [`Tab`].
     pub fn get_active_tab_mut(&mut self) -> Option<&mut Tab> {
         match self.active_tab_index {
@@ -328,6 +333,11 @@ impl Screen {
         }
     }
 
+    /// Returns a mutable reference to this [`Screen`]'s indexed [`Tab`].
+    pub fn get_indexed_tab_mut(&mut self, tab_index: usize) -> Option<&mut Tab> {
+        self.get_tabs_mut().get_mut(&tab_index)
+    }
+
     /// Creates a new [`Tab`] in this [`Screen`], applying the specified [`Layout`]
     /// and switching to it.
     pub fn apply_layout(&mut self, layout: Layout, new_pids: Vec<RawFd>) {
@@ -337,17 +347,17 @@ impl Screen {
             tab_index,
             position,
             String::new(),
-            &self.position_and_size,
+            self.size,
             self.bus.os_input.as_ref().unwrap().clone(),
             self.bus.senders.clone(),
             self.max_panes,
-            None,
             self.mode_info.clone(),
-            self.input_mode,
             self.colors,
             self.session_state.clone(),
+            self.draw_pane_frames,
         );
-        tab.apply_layout(layout, new_pids);
+        tab.apply_layout(layout, new_pids, tab_index);
+        self.tab_history.push(self.active_tab_index);
         self.active_tab_index = Some(tab_index);
         self.tabs.insert(tab_index, tab);
         self.update_tabs();
@@ -392,8 +402,29 @@ impl Screen {
         self.mode_info = mode_info;
         for tab in self.tabs.values_mut() {
             tab.mode_info = self.mode_info.clone();
+            tab.mark_active_pane_for_rerender();
         }
     }
+    pub fn move_focus_left_or_previous_tab(&mut self) {
+        if !self.get_active_tab_mut().unwrap().move_focus_left() {
+            self.switch_tab_prev();
+        }
+    }
+    pub fn move_focus_right_or_next_tab(&mut self) {
+        if !self.get_active_tab_mut().unwrap().move_focus_right() {
+            self.switch_tab_next();
+        }
+    }
+    pub fn toggle_tab(&mut self) {
+        let tab = self.get_previous_tab();
+        if let Some(t) = tab {
+            let position = t.position;
+            self.go_to_tab(position + 1);
+        };
+
+        self.update_tabs();
+        self.render();
+    }
 }
 
 // The box is here in order to make the
@@ -407,22 +438,21 @@ pub(crate) fn screen_thread_main(
     session_state: Arc<RwLock<SessionState>>,
 ) {
     let capabilities = config_options.simplified_ui;
-    let default_mode = config_options.default_mode.unwrap_or_default();
+    let draw_pane_frames = !config_options.no_pane_frames;
 
     let mut screen = Screen::new(
         bus,
         &client_attributes,
         max_panes,
-        ModeInfo {
-            palette: client_attributes.palette,
-            capabilities: PluginCapabilities {
+        get_mode_info(
+            config_options.default_mode.unwrap_or_default(),
+            client_attributes.palette,
+            PluginCapabilities {
                 arrow_fonts: capabilities,
             },
-            mode: default_mode,
-            ..ModeInfo::default()
-        },
-        default_mode,
+        ),
         session_state,
+        draw_pane_frames,
     );
     loop {
         let (event, mut err_ctx) = screen
@@ -508,9 +538,7 @@ pub(crate) fn screen_thread_main(
                 screen.get_active_tab_mut().unwrap().move_focus_left();
             }
             ScreenInstruction::MoveFocusLeftOrPreviousTab => {
-                if !screen.get_active_tab_mut().unwrap().move_focus_left() {
-                    screen.switch_tab_prev();
-                }
+                screen.move_focus_left_or_previous_tab();
                 screen
                     .bus
                     .senders
@@ -524,9 +552,7 @@ pub(crate) fn screen_thread_main(
                 screen.get_active_tab_mut().unwrap().move_focus_right();
             }
             ScreenInstruction::MoveFocusRightOrNextTab => {
-                if !screen.get_active_tab_mut().unwrap().move_focus_right() {
-                    screen.switch_tab_next();
-                }
+                screen.move_focus_right_or_next_tab();
                 screen
                     .bus
                     .senders
@@ -542,12 +568,30 @@ pub(crate) fn screen_thread_main(
                     .unwrap()
                     .scroll_active_terminal_up();
             }
+            ScreenInstruction::ScrollUpAt(point) => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .scroll_terminal_up(&point, 3);
+            }
             ScreenInstruction::ScrollDown => {
                 screen
                     .get_active_tab_mut()
                     .unwrap()
                     .scroll_active_terminal_down();
             }
+            ScreenInstruction::ScrollDownAt(point) => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .scroll_terminal_down(&point, 3);
+            }
+            ScreenInstruction::ScrollToBottom => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .scroll_active_terminal_to_bottom();
+            }
             ScreenInstruction::PageScrollUp => {
                 screen
                     .get_active_tab_mut()
@@ -570,30 +614,17 @@ pub(crate) fn screen_thread_main(
                 screen.get_active_tab_mut().unwrap().close_focused_pane();
                 screen.render();
             }
-            ScreenInstruction::SetSelectable(id, selectable) => {
-                screen
-                    .get_active_tab_mut()
-                    .unwrap()
-                    .set_pane_selectable(id, selectable);
-            }
-            ScreenInstruction::SetFixedHeight(id, fixed_height) => {
-                screen
-                    .get_active_tab_mut()
-                    .unwrap()
-                    .set_pane_fixed_height(id, fixed_height);
-            }
-            ScreenInstruction::SetFixedWidth(id, fixed_width) => {
-                screen
-                    .get_active_tab_mut()
-                    .unwrap()
-                    .set_pane_fixed_width(id, fixed_width);
-            }
-            ScreenInstruction::SetInvisibleBorders(id, invisible_borders) => {
-                screen
-                    .get_active_tab_mut()
-                    .unwrap()
-                    .set_pane_invisible_borders(id, invisible_borders);
-                screen.render();
+            ScreenInstruction::SetSelectable(id, selectable, tab_index) => {
+                screen.get_indexed_tab_mut(tab_index).map_or_else(
+                    || {
+                        log::warn!(
+                            "Tab index #{} not found, could not set selectable for plugin #{:?}.",
+                            tab_index,
+                            id
+                        )
+                    },
+                    |tab| tab.set_pane_selectable(id, selectable),
+                );
             }
             ScreenInstruction::ClosePane(id) => {
                 screen.get_active_tab_mut().unwrap().close_pane(id);
@@ -605,13 +636,12 @@ pub(crate) fn screen_thread_main(
                     .unwrap()
                     .toggle_active_pane_fullscreen();
             }
-            ScreenInstruction::NewTab(pane_id) => {
-                screen.new_tab(pane_id);
-                screen
-                    .bus
-                    .senders
-                    .send_to_server(ServerInstruction::UnblockInputThread)
-                    .unwrap();
+            ScreenInstruction::TogglePaneFrames => {
+                screen.draw_pane_frames = !screen.draw_pane_frames;
+                for (_, tab) in screen.tabs.iter_mut() {
+                    tab.set_pane_frames(screen.draw_pane_frames);
+                }
+                screen.render();
             }
             ScreenInstruction::SwitchTabNext => {
                 screen.switch_tab_next();
@@ -669,9 +699,42 @@ pub(crate) fn screen_thread_main(
                     .toggle_sync_panes_is_active();
                 screen.update_tabs();
             }
+            ScreenInstruction::LeftClick(point) => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .handle_left_click(&point);
+            }
+            ScreenInstruction::MouseRelease(point) => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .handle_mouse_release(&point);
+            }
+            ScreenInstruction::MouseHold(point) => {
+                screen
+                    .get_active_tab_mut()
+                    .unwrap()
+                    .handle_mouse_hold(&point);
+            }
+            ScreenInstruction::Copy => {
+                screen.get_active_tab().unwrap().copy_selection();
+            }
             ScreenInstruction::Exit => {
                 break;
             }
+            ScreenInstruction::ToggleTab => {
+                screen.toggle_tab();
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+            }
         }
     }
 }
+
+#[cfg(test)]
+#[path = "./unit/screen_tests.rs"]
+mod screen_tests;
