@@ -33,6 +33,8 @@ pub enum Event {
     TabUpdate(Vec<TabInfo>),
     KeyPress(Key),
     Timer(f64),
+    CopyToClipboard,
+    InputReceived,
 }
 
 /// Describes the different input modes, which change the way that keystrokes will be interpreted.
@@ -148,6 +150,7 @@ pub struct ModeInfo {
     pub keybinds: Vec<(String, String)>, // <shortcut> => <shortcut description>
     pub palette: Palette,
     pub capabilities: PluginCapabilities,
+    pub session_name: Option<String>,
 }
 
 #[derive(Debug, Default, Clone, PartialEq, Eq, Hash, Deserialize, Serialize)]
